from .. import converters


def test_notebookcontent_to_docxbytes(test_notebook):
    converters.notebookcontent_to_docxbytes(
        test_notebook,
        'download-notebook',
        test_notebook['metadata']['path'],
    )



import os
import re

import matplotlib.pyplot as plt
from nbconvert.preprocessors import ExecutePreprocessor
import nbformat
import numpy as np
import pytest
from pytest_lazyfixture import lazy_fixture
import requests

from ..converters import encode_image_base64


@pytest.fixture(params=['https://nbviewer.jupyter.org/github/unpingco/Python-f'
                        'or-Signal-Processing/blob/master/More_Fourier_Transfo'
                        'rm.ipynb',
                        'https://nbviewer.jupyter.org/github/CamDavidsonPilon/'
                        'Probabilistic-Programming-and-Bayesian-Methods-for-Ha'
                        'ckers/blob/master/Chapter1_Introduction/Ch1_Introduct'
                        'ion_PyMC3.ipynb',
                        'https://nbviewer.jupyter.org/github/waltherg/notebook'
                        's/blob/master/2013-12-03-Crank_Nicolson.ipynb'])
def download_notebook(tmpdir, request):
    notebook_url = request.param

    # check extension of file
    filename = os.path.basename(notebook_url)
    if not notebook_url.endswith('.ipynb'):
        raise ValueError(f'{filename} is not a Jupyter notebook.')

    # redirect notebook if notebook is located on nbviewer.jupyter.org
    if 'nbviewer' in notebook_url:
        r = requests.get(notebook_url)
        link = re.findall('(?<=<a href=").+(?=" title="Download Notebook" '
                          'download>)', r.content.decode('utf8'))
        if len(link) > 0:
            notebook_url = link[0]

    # download notebook
    r = requests.get(notebook_url)

    nb = nbformat.reads(r.content.decode('utf8'), 4)
    nb['metadata'].update({'path': f'{tmpdir}'})

    return nb


@pytest.fixture(params=[10])
def matplotlib_notebook(tmpdir, request):
    nb = nbformat.v4.new_notebook()

    nb.cells.append(nbformat.v4.new_code_cell('\n'.join(
        ['import matplotlib.pyplot as plt',
         '%matplotlib inline'])))

    for _ in range(request.param):
        nb.cells.append(nbformat.v4.new_code_cell('\n'.join(
            ['plt.figure(dpi=100)',
             'plt.plot(range(100))',
             'plt.show()'])))

    ep = ExecutePreprocessor()
    ep.preprocess(nb, {'metadata': {'path': tmpdir}})

    nb['metadata'].update({'path': f'{tmpdir}'})

    return nb


@pytest.fixture(params=['png', 'jpg', 'jpeg'])
def images_notebook(tmpdir, request):
    nb = nbformat.v4.new_notebook()

    filename = 'matplotlib.' + request.param

    plt.figure()
    plt.plot(np.linspace(0, 1), np.power(np.linspace(0, 1), 2))
    plt.savefig(os.path.join(tmpdir, filename))

    # add image as path
    nb.cells.append(nbformat.v4.new_markdown_cell('\n'.join([
        'line1',
        f'![{filename}]({filename})',
        'line3',
    ])))

    # add image as attachment
    nb.cells.append(nbformat.v4.new_markdown_cell('\n'.join([
        'line1',
        f'![{filename}](attachment:{filename})',
        'line3',
    ])))
    nb.cells[-1]['attachments'] = \
        {filename: encode_image_base64(os.path.join(tmpdir, filename))}

    ep = ExecutePreprocessor()
    ep.preprocess(nb, {'metadata': {'path': tmpdir}})

    nb['metadata'].update({'path': f'{tmpdir}'})

    return nb


@pytest.fixture()
def metadata_notebook(tmpdir):
    nb = nbformat.v4.new_notebook()

    ep = ExecutePreprocessor()
    ep.preprocess(nb, {'metadata': {'path': tmpdir}})

    nb['metadata'].update({
        "authors": [
            {"name": "author1"},
            {"name": "author2"}
        ],
        "subtitle": 'subtitle',
        "date": '2019-05-11',
        "path": f"{tmpdir}",
    })

    return nb


@pytest.fixture(params=[
    lazy_fixture('download_notebook'),
    lazy_fixture('matplotlib_notebook'),
    lazy_fixture('images_notebook'),
    lazy_fixture('metadata_notebook')
], ids=[
    'download',
    'matplotlib',
    'embedded-images',
    'metadata',
])
def test_notebook(request):
    return request.param



import base64
import os
from pathlib import Path
import re
import tempfile

from nbconvert import HTMLExporter, preprocessors
import pypandoc
import requests


def encode_image_base64(filepath):
    """Encode an image as a base64 string

    Parameters
    ----------
    filepath : str
        Filepath of the image file

    Returns
    -------
    dict
        Dictionary with identifier as key and base64-encoded data as value.

    """
    key = 'image/' + os.path.splitext(filepath)[1][1:]
    if f'{filepath}'.startswith('http'):
        r = requests.get(filepath)
        data = base64.b64encode(r.content).decode('utf8')
    else:
        with open(filepath, 'rb') as image:
            data = base64.b64encode(image.read()).decode('utf8')

    return {key: data}


def preprocess(content, path):
    """Preprocess the notebook data.
    Cells will specific tags will be removed and attached images will be embedded.

    Parameters
    ----------
    content : nbformat.NotebookNode
        A dict-like node of the notebook with attribute-access
    path : str
        Path to the notebook as string

    Returns
    -------
    content : nbformat.NotebookNode
        Preprocessed notebook content

    """
    tag_preprocessor = preprocessors.TagRemovePreprocessor()
    tag_preprocessor.remove_cell_tags.add('nbconvert-remove-cell')
    tag_preprocessor.remove_input_tags.add('nbconvert-remove-input')
    tag_preprocessor.preprocess(content, {})

    for cell in content['cells']:
        attachment_to_embedded_image(cell)
        linked_to_embedded_image(cell, path)

    return content


def notebook_to_html(content, htmlfile):
    """ Convert notebook to html file.

    Parameters
    ----------

    content : nbformat.NotebookNode
        A dict-like node of the notebook with attribute-access
    htmlfile : str
        Filename for the notebook exported as html
    """
    # prepare html exporter, anchor_link_text=' ' suppress anchors being shown
    html_exporter = HTMLExporter(anchor_link_text=' ',
                                 exclude_input_prompt=True,
                                 exclude_output_prompt=True)

    # export to html
    content, _ = html_exporter.from_notebook_node(content)

    # check if export path exists
    if os.path.dirname(htmlfile) != '' and not os.path.isdir(os.path.dirname(htmlfile)):
        raise FileNotFoundError(f'Path to html-file does not exist: {os.path.dirname(htmlfile)}')

    # write content to html file
    with open(htmlfile, 'w', encoding='utf-8') as file:
        file.write(content)


def html_to_docx(htmlfile, docxfile, handler=None, metadata=None):
    """ Convert html file to docx file.

    Parameters
    ----------

    htmlfile : str
        Filename of the notebook exported as html
    docxfile : str
        Filename for the notebook exported as docx
    handler : tornado.web.RequestHandler, optional
        Handler that serviced the bundle request
    metadata : dict, optional
        Dicts with metadata information of the notebook
    """

    # check if html file exists
    if not os.path.isfile(htmlfile):
        raise FileNotFoundError(f'html-file does not exist: {htmlfile}')

    # check if export path exists
    if os.path.dirname(docxfile) != '' and not os.path.isdir(os.path.dirname(docxfile)):
        raise FileNotFoundError(f'Path to docx-file does not exist: {os.path.dirname(docxfile)}')

    # set extra args for pandoc
    extra_args = []
    if metadata is not None and 'authors' in metadata:
        if isinstance(metadata['authors'], list) and \
                all(['name' in x for x in metadata['authors']]):
            extra_args.append(
                f'--metadata=author:'
                f'{", ".join([x["name"] for x in metadata["authors"]])}')
        elif handler is not None:
            handler.log.warning('Author metadata has wrong format, see https:/'
                                '/github.com/m-rossi/jupyter_docx_bundler/blob'
                                '/master/README.md')
    if metadata is not None and 'subtitle' in metadata:
        extra_args.append(f'--metadata=subtitle:{metadata["subtitle"]}')
    if metadata is not None and 'date' in metadata:
        extra_args.append(f'--metadata=date:{metadata["date"]}')

    # convert to docx
    pypandoc.convert_file(htmlfile,
                          'docx',
                          format='html+tex_math_dollars',
                          outputfile=docxfile,
                          extra_args=extra_args)


def notebookcontent_to_docxbytes(content, filename, path, handler=None):
    """Convert content of a Jupyter notebook to the raw bytes content of a *.docx file

    Parameters
    ----------
    content : nbformat.NotebookNode
        A dict-like node of the notebook with attribute-access
    filename : str
        Filename of the notebook without extension
    handler : tornado.web.RequestHandler, optional
        Handler that serviced the bundle request
    path : str
        Path to the notebook as string
    Returns
    -------
    bytes

    """
    with tempfile.TemporaryDirectory() as tempdir:
        # preprocess notebook
        content = preprocess(content, path)

        # prepare file names
        htmlfile = os.path.join(tempdir, f'{filename}.html')
        docxfile = os.path.join(tempdir, f'{filename}.docx')

        # convert notebook to html
        notebook_to_html(content, htmlfile)

        # convert html to docx
        html_to_docx(
            htmlfile,
            docxfile,
            metadata=content['metadata'],
            handler=handler,
        )

        # read raw data
        with open(docxfile, 'rb') as bundle_file:
            rawdata = bundle_file.read()

        return rawdata


def attachment_to_embedded_image(cell):
    """Converts cell with embedded images as attachments of notebook cell to
    markdown embedded images.

    Parameters
    ----------
    cell : NotebookNode
        Cell with attachments
    """
    if 'attachments' in cell:
        for att in cell['attachments']:
            s = re.split(rf'!\[.+\]\(attachment:{att}\)', cell['source'])
            if len(s) != 2:
                raise NotImplementedError
            for key, val in cell['attachments'][att].items():
                s.insert(1, f'<img src="data:{key};base64,{val}" />')
            cell['source'] = ''.join(s)
        cell.pop('attachments')


def linked_to_embedded_image(cell, path):
    """Converts cell with linked images of notebook cell to markdown embedded images.

    Parameters
    ----------
    cell : NotebookNode
        Cell with attachments
    path : str
        Path to the notebook as string
    """
    path = Path(path)
    re_image = re.compile(rf'!\[.+\)')
    if cell['cell_type'] == 'markdown':
        s = re_image.split(cell['source'])
        images = re_image.findall(cell['source'])
        for ii, image in enumerate(images):
            _, image = image.split('](')
            if image.startswith('http'):
                image = image[:-1]
            elif Path(image[:-1]).is_absolute():
                image = Path(image[:-1])
            else:
                image = (path / Path(image[:-1])).resolve()
            b64 = encode_image_base64(image)
            key = list(b64.keys())[0]
            s.insert(ii + 1, f'<img src="data:{key};base64,{b64[key]}" />')
        cell['source'] = ''.join(s)



import os
import sys

from nbconvert import nbconvertapp
from notebook.config_manager import BaseJSONConfigManager
from jupyter_core.paths import jupyter_config_path


# Check if bundler extension is enabled
bundler_enabled = False
config_dirs = [os.path.join(p, 'nbconfig') for p in jupyter_config_path()]
for config_dir in config_dirs:
    cm = BaseJSONConfigManager(config_dir=config_dir)
    data = cm.get('notebook')
    if 'bundlerextensions' in data:
        for bundler_id, info in data['bundlerextensions'].items():
            label = info.get('label')
            module = info.get('module_name')
            if module == 'jupyter_docx_bundler':
                bundler_enabled = True
if not bundler_enabled:
    sys.exit('jupyter-dox-bundler not enabled.')


# Check if nbconvert lists docx as available format
formats = nbconvertapp.get_export_names()
if 'docx' not in formats:
    sys.exit('*.docx not in nbconvert export-names.')



from pathlib import Path
import os

from mock import patch
from nbconvert import nbconvertapp
import nbformat
from nbformat import write
from nbformat.v4 import (
    new_notebook, new_markdown_cell, new_code_cell, new_output,
)
from notebook.tests.launchnotebook import NotebookTestBase


class BundleAPITest(NotebookTestBase):
    @classmethod
    def setup_class(cls):
        super(BundleAPITest, cls).setup_class()
        nbdir = Path(cls.notebook_dir)

        nb = new_notebook()

        nb.cells.append(new_markdown_cell(u'Created by test'))
        cc1 = new_code_cell(source=u'print(2*6)')
        cc1.outputs.append(new_output(output_type="stream", text=u'12'))
        nb.cells.append(cc1)

        with open(nbdir / 'testnb.ipynb', 'w', encoding='utf-8') as f:
            write(nb, f, version=4)

    def test_bundler_invoke(self):
        with patch('notebook.bundler.handlers.BundlerHandler.get_bundler') as mock:
            mock.return_value = {'module_name': 'jupyter_docx_bundler'}
            resp = self.request('GET', 'bundle/testnb.ipynb',
                                params={'bundler': 'jupyter_docx_bundler'})
            mock.assert_called_with('jupyter_docx_bundler')
        self.assertEqual(resp.status_code, 200)


def test_jupyter_nbconvert_cli(tmpdir, download_notebook):
    ipynb_filename = os.path.join(tmpdir, 'download_notebook.ipynb')
    with open(ipynb_filename, 'w', encoding='utf8') as file:
        file.write(nbformat.writes(download_notebook))

    app = nbconvertapp.NbConvertApp()
    app.initialize(argv=[ipynb_filename, '--to', 'jupyter_docx_bundler.DocxExporter'])
    app.convert_notebooks()



import os
from pathlib import Path

from nbconvert.exporters import Exporter
from tornado import gen

from . import converters


def _jupyter_bundlerextension_paths():
    """ Declare bundler extensions provided by this package."""

    return [{
        # unique bundler name
        "name": "docx_bundler",
        # module containing bundle function
        "module_name": "jupyter_docx_bundler",
        # human-redable menu item label
        "label": "Office Open XML (.docx)",
        # group under 'deploy' or 'download' menu
        "group": "download",
    }]  # pragma: no cover


@gen.coroutine
def bundle(handler, model):
    """ Create a compressed tarball containing the notebook document.

    Parameters
    ----------

    handler : tornado.web.RequestHandler
        Handler that serviced the bundle request
    model : dict
        Notebook model from the configured ContentManager
    """

    # prepare notebook
    if Path(model['path']).is_absolute():
        notebook_path = Path(model['path']).parent
    else:
        notebook_path = (Path.cwd() / model['path']).parent
    notebook_filename = os.path.basename(model['name'])
    notebook_name = os.path.splitext(notebook_filename)[0]

    # Set headers to trigger browser download
    handler.set_header(
        'Content-Disposition',
        f'attachment; filename="{os.path.basename(notebook_name)}.docx"',
    )
    handler.set_header(
        'Content-Type',
        'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    )

    # send content to handler
    yield handler.write(converters.notebookcontent_to_docxbytes(model['content'],
                                                                notebook_filename,
                                                                notebook_path))

    # Return the buffer value as the response
    handler.finish()


class DocxExporter(Exporter):
    """Convert a notebook to docx
    This is the API which nbconvert calls.
    """
    output_mimetype = 'application/docx'

    def _file_extension_default(self):
        return '.docx'

    def from_notebook_node(self, nb, resources=None, **kw):
        nb_copy, resources = super().from_notebook_node(nb, resources)

        return converters.notebookcontent_to_docxbytes(
            nb_copy,
            resources['metadata']['name'],
            resources['metadata']['path'],
        ), resources
